// DataPluggableProtocol.cpp : Implementation of CDataPluggableProtocol

#include "stdafx.h"
#include "DataPluggableProtocol.h"


// CDataPluggableProtocol

//Load data to m_pIStream from DataBase
HRESULT CDataPluggableProtocol::LoadDataToIStream(LPCWSTR wstrUrl)
{
	CString strUrl = wstrUrl;
	CString strFileName;

	// parse URL
	strFileName = strUrl.Right(strUrl.GetLength() - (strUrl.Find(_T("//"),0)+2));

	// open file
	HANDLE hFile = CreateFile(strFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
	_ASSERTE(INVALID_HANDLE_VALUE != hFile);

	// get file size
	DWORD dwFileSize = GetFileSize(hFile, NULL);
	_ASSERTE(-1 != dwFileSize);

	LPVOID pvData = NULL;
	// alloc memory based on file size
	HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE, dwFileSize);
	_ASSERTE(NULL != hGlobal);

	pvData = GlobalLock(hGlobal);
	_ASSERTE(NULL != pvData);

	DWORD dwBytesRead = 0;
	// read file and store in global memory
	BOOL bRead = ReadFile(hFile, pvData, dwFileSize, &dwBytesRead, NULL);
	_ASSERTE(FALSE != bRead);
	GlobalUnlock(hGlobal);
	CloseHandle(hFile);

	// create IStream* from global memory
	m_pIStream.Release();
	HRESULT hr = CreateStreamOnHGlobal(hGlobal, TRUE, &m_pIStream);
	_ASSERTE(SUCCEEDED(hr) && m_pIStream);
	if(FAILED(hr))
		return hr;

	ULARGE_INTEGER nLen;
	LARGE_INTEGER nLen1;
	nLen1.QuadPart = 0;
	if(FAILED(m_pIStream->Seek(nLen1, STREAM_SEEK_END, &nLen)))
		return E_FAIL;
	m_lSize = nLen.LowPart;
	m_lPosition = 0;
	return S_OK;
}


STDMETHODIMP CDataPluggableProtocol::Start(
	LPCWSTR szUrl,
	IInternetProtocolSink *pIProtSink,
	IInternetBindInfo *pIBindInfo,
	DWORD grfSTI,
	DWORD dwReserved)
{
	ATLTRACE(_T("CDataPluggableProtocol::Start ()\n"));

	if (grfSTI & PI_PARSE_URL)
		return S_FALSE;

	// Verify arguments
	if ((szUrl == NULL) || (pIProtSink == NULL))
		return E_INVALIDARG;

	//get connection to database
	if (FAILED(LoadDataToIStream(szUrl)))
		return S_FALSE;

	// Report download progress
	m_pIProtSink = pIProtSink;
	m_pIProtSink->ReportData(BSCF_FIRSTDATANOTIFICATION, 0, m_lSize);

	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::Continue(PROTOCOLDATA *pStateInfo)
{
	ATLTRACE(_T("CDataPluggableProtocol::Continue ()\n"));
	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::Abort(HRESULT hrReason,DWORD dwOptions)
{
	ATLTRACE(_T("CDataPluggableProtocol::Abort ()\n"));
	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::Terminate(DWORD dwOptions)
{
	ATLTRACE(_T("CDataPluggableProtocol::Terminate ()\n"));
	m_pIProtSink.Release();
	m_pIStream.Release();

	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::Suspend()
{
	return E_NOTIMPL;
}

STDMETHODIMP CDataPluggableProtocol::Resume()
{
	return E_NOTIMPL;
}

STDMETHODIMP CDataPluggableProtocol::LockRequest(DWORD dwOptions)
{
	ATLTRACE(_T("CDataPluggableProtocol::LockRequest\n"));
	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::UnlockRequest()
{
	ATLTRACE(_T("CDataPluggableProtocol::UnlockRequest\n"));
	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::Read(void *pv, ULONG cb, ULONG *pcbRead)
{
	BOOL Status = FALSE;

	//set position of begin
	LARGE_INTEGER nLen1;
	nLen1.QuadPart = m_lPosition;

	if(FAILED(m_pIStream->Seek(nLen1, STREAM_SEEK_SET, NULL)))
	{
		return INET_E_DOWNLOAD_FAILURE;
	}

	//read data to browser
	if ( SUCCEEDED( m_pIStream->Read(pv, cb, pcbRead) ))
	{
		if (*pcbRead != 0)
		{
			Status = TRUE;
			m_lPosition += *pcbRead;
		}
		else
			return S_FALSE;
	}

	ATLTRACE(_T("nBuffer is %ld\n"), *pcbRead);

	if (Status == FALSE) 
	{
		DWORD error = GetLastError();
		ATLTRACE(_T("Errno = %d\n"), error);
		return INET_E_DOWNLOAD_FAILURE;
	}
	else
		if (m_lSize - m_lPosition > 0) 	// more to read ?
		{
			if (m_pIProtSink != NULL)
				m_pIProtSink->ReportData(BSCF_INTERMEDIATEDATANOTIFICATION, m_lPosition, m_lSize);
			return S_OK;
		}

		if (m_pIProtSink != NULL)
		{
			m_pIProtSink->ReportData(BSCF_LASTDATANOTIFICATION, m_lSize, m_lSize);
			m_pIProtSink->ReportResult(S_OK, 0, NULL);
		}

	return S_FALSE;	
}

STDMETHODIMP CDataPluggableProtocol::Seek(
	LARGE_INTEGER dlibMove,
	DWORD dwOrigin,
	ULARGE_INTEGER *plibNewPosition)
{
	ATLTRACE(_T("CDataPluggableProtocol::Seek ()\n"));
	return S_OK;
}

STDMETHODIMP CDataPluggableProtocol::CombineUrl(LPCWSTR pwzBaseUrl, LPCWSTR pwzRelativeUrl, DWORD dwCombineFlags,
												LPWSTR pwzResult, DWORD cchResult, DWORD *pcchResult, DWORD dwReserved)
{
	ATLTRACE(_T("CDataPluggableProtocol::CombineUrl (%s, %s)\n"),
		pwzBaseUrl, pwzRelativeUrl);

	return INET_E_DEFAULT_ACTION;
}

STDMETHODIMP CDataPluggableProtocol::CompareUrl(LPCWSTR pwszUrl1, LPCWSTR pwszUrl2, DWORD dwCompareFlags)
{
	ATLTRACE(_T("CDataPluggableProtocol::CompareUrl ()\n"));
	
	if (pwszUrl1 == NULL || pwszUrl2 == NULL)
		return E_POINTER;

	HRESULT hr = S_FALSE;

	if (lstrcmp(pwszUrl1, pwszUrl2) == 0)
	{
		hr = S_OK;
	}

	return hr;
}

STDMETHODIMP CDataPluggableProtocol::ParseUrl(LPCWSTR pwzUrl, PARSEACTION parseAction, DWORD dwParseFlags,
											  LPWSTR pwzResult, DWORD cchResult, DWORD *pcchResult, DWORD dwReserved)
{
	ATLTRACE(_T("CDataPluggableProtocol::ParseUrl (%s, %i)\n"),
				pwzUrl, (int)parseAction);

	return INET_E_DEFAULT_ACTION;
}

STDMETHODIMP CDataPluggableProtocol::QueryInfo( LPCWSTR pwzUrl, QUERYOPTION QueryOption, DWORD dwQueryFlags,
											   LPVOID pBuffer, DWORD cbBuffer, DWORD *pcbBuf, DWORD dwReserved)
{
	ATLTRACE(_T("CDataPluggableProtocol::QueryInfo (%s, %i)\n"),
				pwzUrl, (int)QueryOption);

	return INET_E_DEFAULT_ACTION;
}
